@@ -1,6 +1,6 @@
-/* $Id: sparse_jacobian.cpp 2506 2012-10-24 19:36:49Z bradbell $ */
+/* $Id: sparse_jacobian.cpp 3138 2014-03-02 18:46:11Z bradbell $ */
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -15,24 +15,26 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 /*
 $begin sacado_sparse_jacobian.cpp$$
 $spell
-        bool
-        CppAD
-        sacado
+	const
+	bool
+	CppAD
+	sacado
 	sparse_jacobian
 $$
 
 $section sacado Speed: sparse_jacobian$$
 
 $codep */
-// The sacado version of this test is not yet available
+// A sacado version of this test is not yet available
 bool link_sparse_jacobian(
-	size_t                     size     , 
-	size_t                     repeat   , 
-	size_t                     m        ,
-	CppAD::vector<double>     &x        ,
-	CppAD::vector<size_t>     &i        ,
-	CppAD::vector<size_t>     &j        ,
-	CppAD::vector<double>     &jacobian )
+	size_t                           size     , 
+	size_t                           repeat   , 
+	size_t                           m        ,
+	const CppAD::vector<size_t>&     row      ,
+	const CppAD::vector<size_t>&     col      ,
+	      CppAD::vector<double>&     x        ,
+	      CppAD::vector<double>&     jacobian ,
+	      size_t&                    n_sweep  )
 {
 	return false;
 }
